/**
 * Copyright (c) 2008 Washington University
 */
package org.nrg.xnat.sql;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Iterator;
import java.util.NoSuchElementException;

import org.apache.log4j.Logger;

/**
 * Abstract base class for obtaining value iterators from an SQL query result.
 * @author Kevin A. Archie <karchie@npg.wustl.edu>
 *
 */
public abstract class AbstractResultIterator<V> implements Iterator<V> {
    private final ResultSet rs;
    private final boolean shouldCloseConnection;
    private boolean advanced = false;

    protected AbstractResultIterator(final ResultSet rs, final boolean shouldCloseConnection) {
	this.rs = rs;
	this.shouldCloseConnection = shouldCloseConnection;
    }

    /* (non-Javadoc)
     * @see java.util.Iterator#hasNext()
     */
    public final boolean hasNext() {
	advanced = true;
	try {
	    return rs.next();
	} catch (SQLException e) {
	    return false;
	}
    }

    /* (non-Javadoc)
     * @see java.util.Iterator#next()
     */
    public final V next() {
	try {
	    if (!advanced) {
		final boolean hasNext = rs.next();
		if (!hasNext) {
		    dispose();
		    throw new NoSuchElementException();
		}
	    }
	    advanced = false;
	    if (rs.isAfterLast()) {
		throw new NoSuchElementException();
	    }

	    return makeEntry(rs);
	} catch (SQLException e) {
	    throw new NoSuchElementException("Database error: " + e.getMessage());	    
	}
    }

    /**
     * The ResultSet cursor is on a valid row; make and return a map entry
     */
    protected abstract V makeEntry(final ResultSet rs) throws SQLException;

    /* (non-Javadoc)
     * @see java.util.Iterator#remove()
     */
    public final void remove() { throw new UnsupportedOperationException(); }

    /**
     * Releases the SQL resources associated with this object.
     */
    public final void dispose() {
	final Connection c;
	try {
	    c = shouldCloseConnection ? rs.getStatement().getConnection() : null;
	} catch (SQLException e) {
	    final Logger log = Logger.getLogger(AbstractResultIterator.class);
	    log.error("Can't obtain Connection to close", e);
	    return;
	} finally {
	    try { rs.close(); } catch (SQLException ignore) {}
	}
	if (null != c) {
	    try { c.close(); } catch (SQLException ignore) {}
	}
    }
}
